# Generated by Django 4.2.5 on 2023-09-27 02:09

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_appointment", "0044_alter_appointment_appt_reason_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="appointment",
            name="appt_reason",
            field=models.CharField(
                choices=[
                    ("scheduled", "Scheduled (study-defined)"),
                    ("unscheduled", "Unscheduled / Routine"),
                ],
                help_text="The reason for visit from the visit report will be validated against this response. Refer to the protocol documentation for the definition of a scheduled appointment.",
                max_length=25,
                verbose_name="Reason for appointment",
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="comment",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Comment"
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="appt_reason",
            field=models.CharField(
                choices=[
                    ("scheduled", "Scheduled (study-defined)"),
                    ("unscheduled", "Unscheduled / Routine"),
                ],
                help_text="The reason for visit from the visit report will be validated against this response. Refer to the protocol documentation for the definition of a scheduled appointment.",
                max_length=25,
                verbose_name="Reason for appointment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="comment",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Comment"
            ),
        ),
    ]
