# Generated by Django 4.1.2 on 2023-01-24 15:19

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_appointment", "0034_appointmenttype_alter_appointment_appt_type_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="appointment",
            old_name="appt_type",
            new_name="appt_type_old",
        ),
        migrations.RenameField(
            model_name="historicalappointment",
            old_name="appt_type",
            new_name="appt_type_old",
        ),
        migrations.RenameField(
            model_name="appointment",
            old_name="appt_type_new",
            new_name="appt_type",
        ),
        migrations.RenameField(
            model_name="historicalappointment",
            old_name="appt_type_new",
            new_name="appt_type",
        ),
    ]
