# Generated by Django 4.1.2 on 2022-10-07 03:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_appointment", "0032_alter_appointment_unique_together"),
    ]

    operations = [
        migrations.AlterField(
            model_name="appointment",
            name="appt_reason",
            field=models.CharField(
                choices=[("scheduled", "Routine / Scheduled"), ("unscheduled", "Unscheduled")],
                help_text="The reason for visit from the visit report will be validated against this response. Refer to the protocol documentation for the definition of a scheduled appointment.",
                max_length=25,
                verbose_name="Reason for appointment",
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="appt_timing",
            field=models.CharField(
                choices=[("ontime", "On time (within window period)"), ("missed", "Missed")],
                default="ontime",
                help_text="If late, you may be required to complete a protocol incident report. Refer to the protocol documentation for the allowed window periods of scheduled appointments.",
                max_length=25,
                verbose_name="Timing",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="appt_reason",
            field=models.CharField(
                choices=[("scheduled", "Routine / Scheduled"), ("unscheduled", "Unscheduled")],
                help_text="The reason for visit from the visit report will be validated against this response. Refer to the protocol documentation for the definition of a scheduled appointment.",
                max_length=25,
                verbose_name="Reason for appointment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="appt_timing",
            field=models.CharField(
                choices=[("ontime", "On time (within window period)"), ("missed", "Missed")],
                default="ontime",
                help_text="If late, you may be required to complete a protocol incident report. Refer to the protocol documentation for the allowed window periods of scheduled appointments.",
                max_length=25,
                verbose_name="Timing",
            ),
        ),
    ]
